<?php
/**
 * NOTICE OF LICENSE
 *
 * @author    INVERTUS, UAB www.invertus.eu <support@invertus.eu>
 * @copyright Copyright (c) permanent, INVERTUS, UAB
 * @license   Addons PrestaShop license limitation
 * @see       /LICENSE
 *
 * International Registered Trademark & Property of INVERTUS, UAB
 */

use Invertus\SwedBank\Config\Config;
use Invertus\SwedBank\Service\OrderHandler;
use Invertus\SwedBank\Util\CartValidation;
use Invertus\SwedBank\Util\OrderValidation;
use SwedbankPaymentPortal\PayPal\CommunicationEntity\SetExpressCheckoutRequest\Transaction\PayPalTxn;
use SwedbankPaymentPortal\PayPal\CommunicationEntity\SetExpressCheckoutRequest\SetExpressCheckoutRequest;
use SwedbankPaymentPortal\PayPal\CommunicationEntity\SetExpressCheckoutRequest\Transaction;
use SwedbankPaymentPortal\PayPal\CommunicationEntity\SetExpressCheckoutRequest\Transaction\TxnDetails;
use SwedbankPaymentPortal\PayPal\CommunicationEntity\ShippingAddress;
use SwedbankPaymentPortal\PayPal\Type\PayPalBool;
use SwedbankPaymentPortal\SharedEntity\Amount;
use SwedbankPaymentPortal\SwedbankPaymentPortal;
use SwedbankPaymentPortal\UrlCallback;

class SwedBankPayPalModuleFrontController extends ModuleFrontController
{
    const FILENAME = 'paypal';
    /**
     * @var SwedBank
     */
    public $module;

    public function postProcess()
    {
        $idLang = $this->context->language->id;
        $idShop = $this->context->shop->id;

        /** @var SwedbankPaymentPortal $paymentPortal */
        $paymentPortal = $this->module->get(SwedbankPaymentPortal::class);

        /** @var OrderHandler $orderService */
        $orderService = $this->module->get(OrderHandler::class);
        $environment = $this->module->getEnvironment();
        $auth = $orderService->getAuthentication($environment);
        $paymentName = $this->module->l('PayPal', self::FILENAME);

        if (Tools::isSubmit('id_order')) {
            $idOrder = Tools::getValue('id_order');
        } else {
            $idOrder = $orderService->processOrderCreate($this->context->cart, $this->context->currency, $paymentName);
        }

        $order = new Order($idOrder);

        if ($paymentName != $order->payment) {
            $order->payment = $paymentName;
            $order->save();
        }

        $orderConfirmUrl = $orderService->getOrderConfirmationUrl($order, $idLang);

        if (!Validate::isLoadedObject($order)) {
            $this->errors[] = $this->module->l('Order is not valid.', self::FILENAME);
            $this->redirectWithNotifications($orderConfirmUrl);
        }

        if (!$orderService->checkCustomerAccessByOrder($order, $this->context->customer)) {
            $this->errors[] = $this->module->l('Order is not valid.', self::FILENAME);
            $this->redirectWithNotifications($orderConfirmUrl);
        }

        $orderAmount = $orderService->getOrderAmount($order);
        if (false === $orderAmount) {
            $this->errors[] = $this->module->l('incorrect currency.', self::FILENAME);
            $this->redirectWithNotifications($orderConfirmUrl);
        }

        if (!$orderService->validatePrice($orderAmount)) {
            $this->errors[] = $this->module->l('Invalid order price.', self::FILENAME);
            $this->redirectWithNotifications($orderConfirmUrl);
        }

        $customer = $this->context->customer;
        $urlParams = ['id_order' => $idOrder];

        $confirmUrl = $this->context->link->getModuleLink($this->module->name, 'paypalconfirm', $urlParams);
        $cancelUrl = $orderService->getOrderDetailLink($customer, $order, $idLang);

        $idUnique = Tools::passwdGen(4);
        SwedbankOrderPayment::associateUniqueId($order->id, $idShop, $idUnique);

        $merchantReference = $orderService->formatMerchantReference($order, $idShop);

        $amount = new Amount($orderAmount);
        $shippingAddress = $this->getShippingAddress();
        $payPalTxn = new PayPalTxn(
            null,
            '',
            $this->module->l($this->context->shop->name),
            $customer->email,
            $merchantReference,
            $shippingAddress->getCountryCode(),
            $amount->getValue(),
            PayPalBool::false(),
            PayPalBool::false(),
            PayPalBool::false(),
            $confirmUrl,
            $cancelUrl
        );

        $payPalTxn->setMethod('set_express_checkout');

        $txnDetails = new TxnDetails($amount, $merchantReference);
        $transaction = new Transaction($txnDetails, $payPalTxn);
        $purchaseRequest = (new SetExpressCheckoutRequest($transaction, $auth));

        $callbackParams = ['id_order' => $order->id, 'secret_key' => Tools::encrypt($this->module->name)];
        $callbackUrl = $this->context->link->getModuleLink($this->module->name, 'paypalcallback', $callbackParams);
        $transactionHandler = UrlCallback::create($callbackUrl);

        try {
            $response = $paymentPortal->getPayPalGateway()
                ->initPayment($purchaseRequest, $transactionHandler);

            $isLive = Configuration::get(Config::LIVE_ENVIRONMENT);
            $redirectUrl = $response->getCustomerRedirectUrl($isLive);

            Tools::redirect($redirectUrl);
        } catch (Exception $e) {
            $this->errors[] = $this
                ->module->l('Operation failed. Please choose another payment method or try again.', self::FILENAME);
            $this->redirectWithNotifications($orderConfirmUrl);
        }
    }

    private function getShippingAddress()
    {
        $address = new Address($this->context->cart->id_address_delivery, $this->context->language->id);
        $country = new Country($address->id_country, $this->context->language->id, $this->context->shop->id);
        $state = new State($address->id_state, $this->context->language->id, $this->context->shop->id);
        $stateCode = '';
        if (Validate::isLoadedObject($state)) {
            $stateCode = $state->iso_code;
        }
        $shippingAddress = new ShippingAddress(
            $address->city,
            $country->iso_code,
            $address->firstname.' '.$address->lastname,
            $address->postcode,
            $stateCode,
            $address->address1,
            $address->address2,
            ($address->phone_mobile) ? $address->phone_mobile : $address->phone
        );
        return $shippingAddress;
    }

    public function checkAccess()
    {
        $module = $this->module;
        $customer = $this->context->customer;

        $isPaymentProcess = Tools::isSubmit('id_order');

        $link = $this->context->link;
        $redirectUrl = $isPaymentProcess ?
            $link->getPageLink('history') :
            $link->getPageLink('order');

        if (!$module->active || !$module->isConfigured()) {
            Tools::redirect($redirectUrl);
        }

        $idOrder = Tools::getValue('id_order');
        $order = new Order($idOrder);

        if ($isPaymentProcess) {
            /** @var OrderValidation $orderValidation */
            $orderValidation = $module->get(OrderValidation::class);
            $isAccessGranted = $orderValidation->validate($order, $customer);
        } else {
            /** @var CartValidation $cartValidation */
            $cartValidation = $module->get(CartValidation::class);
            $isAccessGranted = $cartValidation->validate($this->context->cart, $this->context->customer);
        }

        if (!$isAccessGranted) {
            Tools::redirect($redirectUrl);
        }

        return true;
    }
}
