<?php
/**
 * NOTICE OF LICENSE
 *
 * @author    INVERTUS, UAB www.invertus.eu <support@invertus.eu>
 * @copyright Copyright (c) permanent, INVERTUS, UAB
 * @license   Addons PrestaShop license limitation
 * @see       /LICENSE
 *
 * International Registered Trademark & Property of INVERTUS, UAB
 */

namespace Invertus\SwedBank\Install;

use Db;
use Exception;
use Invertus\SwedBank\Adapter\Language;
use Invertus\SwedBank\Adapter\Tab;
use OrderState;
use SwedBank;
use Invertus\SwedBank\Adapter\Configuration;
use Tools;

class Installer
{
    /**
     * @var SwedBank
     */
    private $module;

    /**
     * @var Configuration
     */
    private $configuration;

    /**
     * @var Language
     */
    private $language;

    /**
     * @var Tab
     */
    private $tab;

    /**
     * @var array
     */
    private $moduleConfiguration;

    public function __construct(
        SwedBank $module,
        Configuration $configuration,
        Language $language,
        Tab $tab,
        array $config
    ) {
        $this->module = $module;
        $this->configuration = $configuration;
        $this->language = $language;
        $this->tab = $tab;
        $this->moduleConfiguration = $config;
    }

    public function install()
    {
        if (!$this->registerHooks()) {
            return false;
        }

        if (!$this->installConfiguration()) {
            return false;
        }

        if (!$this->installTabs()) {
            return false;
        }
        
        if (!$this->installDb()) {
            return false;
        }

        if (!$this->createLogFile()) {
            return false;
        }

        return true;
    }
    /**
     * Uninstall module
     *
     * @return bool
     */
    public function uninstall()
    {
        if (!$this->uninstallConfiguration()) {
            return false;
        }

        if (!$this->uninstallTabs()) {
            return false;
        }
        
        if (!$this->uninstallDb()) {
            return false;
        }

        if (!$this->deleteLogFile()) {
            return false;
        }

        return true;
    }
    /**
     * Register module hooks
     *
     * @return bool
     */
    protected function registerHooks()
    {
        $hooks = $this->moduleConfiguration['hooks'];

        foreach ($hooks as $hookName) {
            if (!$this->module->registerHook($hookName)) {
                return false;
            }
        }

        return true;
    }

    protected function installConfiguration()
    {
        $configuration = $this->moduleConfiguration['configuration'];

        foreach ($configuration as $name => $value) {
            if (false !== strpos($name, 'ORDER_STATUS')) {
                $value = $this->configuration->get($value);
            }

            if (!$this->configuration->set($name, $value)) {
                return false;
            }
        }

        return true;
    }

    protected function uninstallConfiguration()
    {
        $configurationNames = array_keys($this->moduleConfiguration['configuration']);

        foreach ($configurationNames as $name) {
            if (!$this->configuration->remove($name)) {
                return false;
            }
        }

        return true;
    }

    protected function installTabs()
    {
        $tabs = $this->moduleConfiguration['tabs'];
        $languages = $this->language->getIds();

        foreach ($tabs as $tab) {
            if ($this->tab->getIdFromClassName($tab['class_name'])) {
                continue;
            }

            $moduleTab = new \Tab();

            foreach ($languages as $idLang) {
                $langIsoCode = \Language::getIsoById($idLang);

                if (isset($tab['name'][$langIsoCode])) {
                    $name = $tab['name'][$langIsoCode];
                } else {
                    $name = $tab['name']['en'];
                }

                $moduleTab->name[$idLang] = $name;
            }

            $moduleTab->class_name = $tab['class_name'];
            $moduleTab->id_parent = is_int($tab['parent_class_name']) ?
                $tab['parent_class_name'] :
                $this->tab->getIdFromClassName($tab['parent_class_name']);
            
            $moduleTab->module = $this->module->name;

            if (!$moduleTab->save()) {
                return false;
            }
        }

        return true;
    }

    protected function uninstallTabs()
    {
        $tabs = $this->moduleConfiguration['tabs'];

        foreach ($tabs as $tab) {
            $tabId = (int) $this->tab->getIdFromClassName($tab['class_name']);

            if (!$tabId) {
                continue;
            }

            $tab = new \Tab($tabId);

            if (!$tab->delete()) {
                return false;
            }
        }

        return true;
    }

    /**
     * Install database
     *
     * @return bool
     *
     * @throws Exception
     */
    private function installDb()
    {
        $installSqlFiles = glob($this->module->getLocalPath().'sql/install/*.sql');

        if (empty($installSqlFiles)) {
            return true;
        }

        foreach ($installSqlFiles as $sqlFile) {
            $sqlStatements = $this->getSqlStatements($sqlFile);

            if (!$this->execute($sqlStatements)) {
                throw new Exception(sprintf('Failed to execute SQL in file `%s`', $sqlFile));
            }
        }

        return true;
    }

    /**
     * Uninstall database
     *
     * @return bool
     *
     * @throws Exception
     */
    private function uninstallDb()
    {
        $uninstallSqlFileName = $this->module->getLocalPath().'sql/uninstall/uninstall.sql';

        if (!file_exists($uninstallSqlFileName)) {
            return true;
        }

        $sqlStatements = $this->getSqlStatements($uninstallSqlFileName);

        return (bool) $this->execute($sqlStatements);
    }

    /**
     * Execute SQL statements
     *
     * @param $sqlStatements
     *
     * @return bool
     */
    private function execute($sqlStatements)
    {
        try {
            $result = Db::getInstance()->execute($sqlStatements);
        } catch (Exception $e) {
            return false;
        }

        return (bool) $result;
    }

    /**
     * Format and get sql statements from file
     *
     * @param string $fileName
     *
     * @return string
     */
    private function getSqlStatements($fileName)
    {
        $sqlStatements = Tools::file_get_contents($fileName);
        $sqlStatements = str_replace('PREFIX_', _DB_PREFIX_, $sqlStatements);
        $sqlStatements = str_replace('ENGINE_TYPE', _MYSQL_ENGINE_, $sqlStatements);

        return $sqlStatements;
    }

    private function createLogFile()
    {
        $logFileDir = $this->module->getLocalPath().'var/logs/swedbank.log';
        if (file_exists($logFileDir)) {
            return true;
        }

        $logFile = fopen($logFileDir, 'w');

        if (false === $logFile) {
            return false;
        }

        $text =
            sprintf(
                $this->module->l('File created for module %s at %s'),
                $this->module->name,
                date('Y-m-d H:i:s')
            );

        if (false === fwrite($logFile, $text)) {
            return false;
        }

        return fclose($logFile);
    }

    private function deleteLogFile()
    {
        $logFilePath = $this->module->getLocalPath().'var/logs/swedbank.log';

        if (!file_exists($logFilePath)) {
            return true;
        }

        if (!unlink($logFilePath)) {
            return false;
        }

        return true;
    }
}
