<?php
/**
 * NOTICE OF LICENSE
 *
 * @author    INVERTUS, UAB www.invertus.eu <support@invertus.eu>
 * @copyright Copyright (c) permanent, INVERTUS, UAB
 * @license   Addons PrestaShop license limitation
 * @see       /LICENSE
 *
 * International Registered Trademark & Property of INVERTUS, UAB
 */

use Invertus\SwedBank\Config\Config;
use Invertus\SwedBank\Install\Installer;
use Invertus\SwedBank\Service\ErrorHandler;
use Invertus\SwedBank\Service\Logger;
use Invertus\SwedBank\Service\OrderHandler;
use Invertus\SwedBank\Service\PaymentOptionProvider;
use SwedbankPaymentPortal\SwedbankPaymentPortal;
use Symfony\Component\Config\FileLocator;
use Symfony\Component\DependencyInjection\ContainerBuilder;
use Symfony\Component\DependencyInjection\Loader\YamlFileLoader;

class SwedBank extends PaymentModule
{
    const ENV_TEST = 'test';
    const ENV_LIVE = 'live';

    const ADMIN_CONFIGURATION_CONTROLLER = 'AdminSwedBankConfiguration';
    const ADMIN_LIVE_GENERAL_SETTINGS_CONTROLLER = 'AdminSwedBankLiveGeneralSettings';
    const ADMIN_TEST_GENERAL_SETTINGS_CONTROLLER = 'AdminSwedBankTestGeneralSettings';
    const ADMIN_LOG_FILE_DISPLAY_CONTROLLER = 'AdminSwedBankLogFileDisplay';

    /**
     * @var ContainerBuilder
     */
    private $container;

    public function __construct()
    {
        $this->name = 'swedbank';
        $this->tab = 'payments_gateways';
        $this->version = '1.0.1';
        $this->author = 'Invertus';

        $this->ps_versions_compliancy = $this->getVersionRange();

        parent::__construct();

        $this->autoload();
        $this->compile();

        $this->displayName = $this->l('Swedbank');
        $this->description = $this->l('Accept payments via Banklink, Credit card and PayPal.');
    }

    public function getContent()
    {
        Tools::redirectAdmin($this->context->link->getAdminLink('AdminSwedBankConfiguration'));
    }

    public function install()
    {
        $currentPhpVersion = Tools::checkPhpVersion();

        if (Tools::version_compare($currentPhpVersion, '5.6.0')) {
            $message = sprintf(
                $this->l('Your PHP version is not supported for module %s. Minimum required PHP version is 5.6.0'),
                $this->displayName
            );

            throw new RuntimeException($message);
        }

        /** @var Installer $installer */
        $installer = $this->container->get(Installer::class);

        return parent::install() && $installer->install();
    }

    public function uninstall()
    {
        /** @var Installer $installer */
        $installer = $this->container->get(Installer::class);

        return $installer->uninstall() && parent::uninstall();
    }

    public function get($id)
    {
        return $this->container->get($id);
    }

    /**
     * Get environment depending on configured settings
     *
     * @return string
     */
    public function getEnvironment()
    {
        $isLiveMode = (bool) Configuration::get('SB_LIVE_ENVIRONMENT');
        $isTestMode = (bool) Configuration::get('SB_TEST_ENVIRONMENT');

        if ($isLiveMode === $isTestMode) {
            throw new RuntimeException('Both Test and Live modes are either enabled or disabled.');
        }

        return $isLiveMode ? self::ENV_LIVE : self::ENV_TEST;
    }

    /**
     * Check if module is configured
     *
     * @return bool
     */
    public function isConfigured()
    {
        $env = Tools::strtoupper($this->getEnvironment());

        $clientId = Configuration::get('SB_CLIENT_ID_'.$env);
        $clientPassword = Configuration::get('SB_CLIENT_PASSWORD_'.$env);

        if (empty($clientId) || empty($clientPassword)) {
            return false;
        }

        $euro = Currency::getIdByIsoCode('EUR', $this->context->shop->id);

        if (!$euro) {
            return false;
        }

        return true;
    }

    public function hookActionFrontControllerSetMedia()
    {
        $controller = $this->context->controller->php_self;
        if (in_array($controller, ['order-detail', 'guest-tracking'])) {
            $this->context->controller->registerJavascript(
                'swedbank-front',
                'modules/'.$this->name.'/views/js/front/front.js'
            );
        }
    }

    /**
     * Display payment options in Front Office checkout
     *
     * @return array
     */
    public function hookPaymentOptions()
    {
        if (!$this->active || !$this->isConfigured()) {
            return [];
        }

        $cartCurrency = new Currency($this->context->cart->id_currency);
        if ('EUR' != $cartCurrency->iso_code) {
            return [];
        }

        $environment = $this->getEnvironment();

        /** @var PaymentOptionProvider $paymentOptionProvider */
        $paymentOptionProvider = $this->get(PaymentOptionProvider::class);
        $paymentOptions = $paymentOptionProvider->getPaymentOptions($environment);

        return $paymentOptions;
    }

    public function hookDisplayOrderConfirmation()
    {
        $idOrder = (int) Tools::getValue('id_order');
        $order = new Order($idOrder, $this->context->language->id);
        if (!Validate::isLoadedObject($order)) {
            return;
        }

        $customer = $this->context->customer;
        if ($customer->secure_key !== $order->getCustomer()->secure_key) {
            return;
        }

        if ($order->module !== $this->name) {
            return;
        }

        if ($order->hasBeenPaid()) {
            $this->context->controller->success[] =
                sprintf($this->l('Your order %s is completed. Payment has been accepted.'), $order->reference);
        } else {
            $this->context->controller->info[] =
                sprintf($this->l('Your order %s is processing.'), $order->reference);
        }
    }

    /**
     * Display "Continue with payment" button in order details
     *
     * @param array $params
     * @return string
     */
    public function hookDisplayOrderDetail(array $params)
    {
        /** @var Order $order */
        $order = $params['order'];

        if ($order->module != $this->name || $order->hasBeenPaid() || !$this->isConfigured() || !$this->active) {
            return '';
        }

        $paymentContinueStates = Configuration::getMultiple([
            'SB_ORDER_STATUS_AWAITING_LIVE',
            'SB_ORDER_STATUS_AWAITING_TEST',
        ]);

        $idCurrentState = $order->getCurrentState();
        if (!in_array($idCurrentState, $paymentContinueStates)) {
            return '';
        }

        $params = ['id_order' => $order->id];

        $environment = $this->getEnvironment();

        if ($order->isAssociatedAtGuest(Tools::getValue('email'))) {
            $customer = new Customer($order->id_customer);

            $customer->logged = 1;
            $this->context->customer = $customer;
            $this->context->cookie->id_customer = (int) $customer->id;
            $this->context->cookie->customer_lastname = $customer->lastname;
            $this->context->cookie->customer_firstname = $customer->firstname;
            $this->context->cookie->logged = 1;
            $this->context->cookie->check_cgv = 1;
            $this->context->cookie->is_guest = $customer->isGuest();
            $this->context->cookie->passwd = $customer->passwd;
            $this->context->cookie->email = $customer->email;
        }

        /** @var PaymentOptionProvider $paymentOptionProvider */
        $paymentOptionProvider = $this->get(PaymentOptionProvider::class);
        $paymentOptions = $paymentOptionProvider->getPaymentOptions($environment, $params);

        $this->context->smarty->assign([
            'paymentOptions' => $paymentOptions,
            'isGuestTrackingPage' => 'guest-tracking' === $this->context->controller->php_self ? true : false
        ]);

        return $this->context->smarty->fetch(
            $this->getLocalPath().'views/templates/hook/displayOrderDetail.tpl'
        );
    }

    public function hookActionGetExtraMailTemplateVars(array &$params)
    {
        $template = $params['template'];
        if ('payment' != $template) {
            return;
        }

        $idOrder = $params['template_vars']['{id_order}'];
        $order = new Order($idOrder);

        if (!Validate::isLoadedObject($order) || $order->module != $this->name) {
            return;
        }

        $params['extra_template_vars']['{swedbank_html_block}'] = '';
        $params['extra_template_vars']['{swedbank_txt_block}'] = '';

        $cardData = new PrestaShopCollection('SwedbankCardPaymentData');
        $cardData->where('id_order', '=', $order->id);
        $cardData = $cardData->getFirst();

        if (!$cardData instanceof SwedbankCardPaymentData) {
            return;
        }
        $tplVars = [
            'pan' => $cardData->pan,
            'expiry_date' => $cardData->expiry_date,
            'authorization_code' => $cardData->authorization_code,
            'merchant_reference' => $cardData->merchant_reference,
            'fullfil_date' => $cardData->fulfill_date,
        ];

        $this->context->smarty->assign($tplVars);

        $params['extra_template_vars']['{swedbank_html_block}'] = $this->context->smarty->fetch(
            $this->getLocalPath().'views/templates/hook/actionGetExtraMailTemplateVars.tpl'
        );
        $params['extra_template_vars']['{swedbank_txt_block}'] = $this->context->smarty->fetch(
            $this->getLocalPath().'views/templates/hook/actionGetExtraMailTemplateVars.txt'
        );
    }

    public function hookActionPaymentCCAdd(&$params)
    {
        /** @var OrderPayment $paymentCC */
        $paymentCC = $params['paymentCC'];
        $orders = Order::getByReference($paymentCC->order_reference);
        if (empty($orders)) {
            return;
        }
        /** @var Order $order */
        foreach ($orders as $order) {
            if ($this->name !== $order->module) {
                continue;
            }

            $idUnique = SwedbankOrderPayment::retrieveUniqueId($order->id, $this->context->shop->id);
            /** @var OrderHandler $orderHandler */
            $orderHandler = $this->get(OrderHandler::class);
            $idOrderPayment = $orderHandler->getMerchantReference($order, $idUnique);
            $paymentCC->transaction_id = $idOrderPayment;
            $paymentCC->payment_method = $order->payment;
            $paymentCC->update();
            break;
        }
    }

    private function autoload()
    {
        require_once dirname(__FILE__).'/vendor/autoload.php';
    }

    private function compile()
    {
        $this->container = new ContainerBuilder();

        $locator = new FileLocator($this->getLocalPath().'config');

        $loader  = new YamlFileLoader($this->container, $locator);
        $loader->load('config.yml');

        $this->container->compile();
    }

    public function initCronTasks()
    {
        if (!$this->isConfigured()) {
            return;
        }

        try {
            /** @var SwedbankPaymentPortal $paymentPortal */
            $paymentPortal = $this->get(SwedbankPaymentPortal::class);
            $paymentPortal->getPayPalGateway()->checkPendingTransactions();
            $paymentPortal->getBankLinkGateway()->checkPendingTransactions();
            $paymentPortal->getPaymentCardHostedPagesGateway()->checkPendingTransactions();
        } catch (Exception $e) {
            $environment = Tools::strtoupper($this->getEnvironment());
            $debugMode = (bool) Configuration::get('SB_DEBUG_MODE_'.$environment);
            if ($debugMode) {
                $logger = new Logger($this);
                $logger->logSimpleData(
                    $this->l('Cronjob error'),
                    $e->getMessage()
                );
                die($this->l('Cronjob error occurred. Please see swedbank.log file for more details'));
            }
            die(
                $this->l('Cronjob failed. Please enable debug mode in order to see more details about error in next cronjob iteration')
            );
        }
    }

    /**
     * Gets Prestashop versions. This comparison is required because Prestashop, from 1.7.4.0 uses 3.4
     * dependency injection package but lower Prestashops versions uses 2.8 symfony dependency injection package. To
     * avoid conflicts with swedbank library, symfony packages must match the Prestashop's currently used versions.
     *
     * @return array
     */
    private function getVersionRange()
    {
        $currentVersion = _PS_VERSION_;
        $isGreaterOrEqual = Tools::version_compare($currentVersion, '1.7.4.0', '>=');

        $minVersion = $isGreaterOrEqual ? $currentVersion : '1.7.1.0';
        $maxVersion = $isGreaterOrEqual ? $currentVersion : '1.7.4.0';

        return [
            'min' => $minVersion,
            'max' => $maxVersion
        ];
    }
}
